@@ -77,7 +77,8 @@ gdk_pango_context_get_info (PangoContext *context, gboolean create)
 }
 
 static GdkGC *
-gdk_pango_get_gc (PangoContext   *context,
+gdk_pango_get_gc (GdkDrawable    *drawable,
+		  PangoContext   *context,
 		  PangoColor     *fg_color,
                   GdkBitmap      *stipple,
 		  GdkGC          *base_gc)
@@ -95,7 +96,7 @@ gdk_pango_get_gc (PangoContext   *context,
       return NULL;
     }
 
-  result = gdk_gc_new (gdk_get_default_root_window ());
+  result = gdk_gc_new (drawable);
   gdk_gc_copy (result, base_gc);
   
   if (fg_color)
@@ -249,7 +250,7 @@ gdk_draw_layout_line_with_colors (GdkDrawable      *drawable,
               tmp.green = background->green;
             }
           
-          bg_gc = gdk_pango_get_gc (context, &tmp, stipple, gc);
+          bg_gc = gdk_pango_get_gc (drawable, context, &tmp, stipple, gc);
           
 	  gdk_draw_rectangle (drawable, bg_gc, TRUE,
 			      x + (x_off + logical_rect.x) / PANGO_SCALE,
@@ -274,7 +275,7 @@ gdk_draw_layout_line_with_colors (GdkDrawable      *drawable,
               tmp.green = foreground->green;
             }
           
-          fg_gc = gdk_pango_get_gc (context, fg_set ? &tmp : NULL,
+          fg_gc = gdk_pango_get_gc (drawable, context, fg_set ? &tmp : NULL,
                                     stipple, gc);
         }
       else
@@ -290,7 +291,7 @@ gdk_draw_layout_line_with_colors (GdkDrawable      *drawable,
           if (embossed)
             {
               PangoColor color = { 65535, 65535, 65535 };
-              GdkGC *white_gc = gdk_pango_get_gc (context, &color, stipple, fg_gc);
+              GdkGC *white_gc = gdk_pango_get_gc (drawable, context, &color, stipple, fg_gc);
               gdk_draw_glyphs (drawable, white_gc, run->item->analysis.font,
                                gx + 1,
                                gy + 1,
