@@ -44,7 +44,7 @@
 #define SPIN_BUTTON_INITIAL_TIMER_DELAY    200
 #define SPIN_BUTTON_TIMER_DELAY            20
 #define MAX_TIMER_CALLS                    5
-#define EPSILON                            1e-5
+#define EPSILON                            1e-10
 #define	MAX_DIGITS			   20
 
 enum {
@@ -72,6 +72,7 @@ static void gtk_spin_button_class_init     (GtkSpinButtonClass *klass);
 static void gtk_spin_button_editable_init  (GtkEditableClass   *iface);
 static void gtk_spin_button_init           (GtkSpinButton      *spin_button);
 static void gtk_spin_button_finalize       (GObject            *object);
+static void gtk_spin_button_destroy        (GtkObject          *object);
 static void gtk_spin_button_set_property   (GObject         *object,
 					    guint            prop_id,
 					    const GValue    *value,
@@ -102,9 +103,14 @@ static gint gtk_spin_button_leave_notify   (GtkWidget          *widget,
 					    GdkEventCrossing   *event);
 static gint gtk_spin_button_focus_out      (GtkWidget          *widget,
 					    GdkEventFocus      *event);
+static void gtk_spin_button_grab_notify    (GtkWidget          *widget,
+					    gboolean            was_grabbed);
+static void gtk_spin_button_state_changed  (GtkWidget          *widget,
+					    GtkStateType        previous_state);
 static void gtk_spin_button_draw_arrow     (GtkSpinButton      *spin_button, 
 					    guint               arrow);
 static gint gtk_spin_button_timer          (GtkSpinButton      *spin_button);
+static void gtk_spin_button_stop_spinning  (GtkSpinButton      *spin);
 static void gtk_spin_button_value_changed  (GtkAdjustment      *adjustment,
 					    GtkSpinButton      *spin_button); 
 static gint gtk_spin_button_key_press      (GtkWidget          *widget,
@@ -186,6 +192,8 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
   gobject_class->set_property = gtk_spin_button_set_property;
   gobject_class->get_property = gtk_spin_button_get_property;
 
+  object_class->destroy = gtk_spin_button_destroy;
+
   widget_class->map = gtk_spin_button_map;
   widget_class->unmap = gtk_spin_button_unmap;
   widget_class->realize = gtk_spin_button_realize;
@@ -202,6 +210,8 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
   widget_class->enter_notify_event = gtk_spin_button_enter_notify;
   widget_class->leave_notify_event = gtk_spin_button_leave_notify;
   widget_class->focus_out_event = gtk_spin_button_focus_out;
+  widget_class->grab_notify = gtk_spin_button_grab_notify;
+  widget_class->state_changed = gtk_spin_button_state_changed;
 
   entry_class->activate = gtk_spin_button_activate;
 
@@ -382,7 +392,7 @@ gtk_spin_button_get_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_ADJUSTMENT:
-      g_value_set_object (value, G_OBJECT (spin_button->adjustment));
+      g_value_set_object (value, spin_button->adjustment);
       break;
     case PROP_CLIMB_RATE:
       g_value_set_double (value, spin_button->climb_rate);
@@ -444,6 +454,14 @@ gtk_spin_button_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gtk_spin_button_destroy (GtkObject *object)
+{
+  gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (object));
+  
+  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+}
+
 static void
 gtk_spin_button_map (GtkWidget *widget)
 {
@@ -572,7 +590,7 @@ gtk_spin_button_size_request (GtkWidget      *widget,
   GtkEntry *entry;
   GtkSpinButton *spin_button;
   gint arrow_size;
-  
+
   g_return_if_fail (requisition != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
@@ -739,6 +757,7 @@ gtk_spin_button_draw_arrow (GtkSpinButton *spin_button,
   gint arrow_size;
 
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+  g_return_if_fail (arrow == GTK_ARROW_UP || arrow == GTK_ARROW_DOWN);
   
   widget = GTK_WIDGET (spin_button);
   spin_shadow_type = spin_button_get_shadow_type (spin_button);
@@ -891,6 +910,22 @@ gtk_spin_button_focus_out (GtkWidget     *widget,
   return GTK_WIDGET_CLASS (parent_class)->focus_out_event (widget, event);
 }
 
+static void
+gtk_spin_button_grab_notify (GtkWidget *widget,
+			     gboolean   was_grabbed)
+{
+  if (!was_grabbed)
+    gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (widget));
+}
+
+static void
+gtk_spin_button_state_changed (GtkWidget    *widget,
+			       GtkStateType  previous_state)
+{
+  if (!GTK_WIDGET_IS_SENSITIVE (widget))
+    gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (widget));    
+}
+
 static gint
 gtk_spin_button_scroll (GtkWidget      *widget,
 			GdkEventScroll *event)
@@ -920,6 +955,42 @@ gtk_spin_button_scroll (GtkWidget      *widget,
   return TRUE;
 }
 
+static void
+gtk_spin_button_stop_spinning(GtkSpinButton *spin)
+{
+  if (spin->timer)
+    {
+      gtk_timeout_remove (spin->timer);
+      spin->timer = 0;
+      spin->timer_calls = 0;
+      spin->need_timer = FALSE;
+    }
+
+  spin->button = 0;
+  spin->timer = 0;
+}
+
+static void
+start_spinning (GtkSpinButton *spin,
+		GtkArrowType   click_child,
+		gfloat         step)
+{
+  g_return_if_fail (click_child == GTK_ARROW_UP || click_child == GTK_ARROW_DOWN);
+  
+  spin->click_child = click_child;
+  gtk_spin_button_real_spin (spin, click_child == GTK_ARROW_UP ? step : -step);
+  
+  if (!spin->timer)
+    {
+      spin->timer_step = step;
+      spin->need_timer = TRUE;
+      spin->timer = gtk_timeout_add (SPIN_BUTTON_INITIAL_TIMER_DELAY, 
+				     (GtkFunction) gtk_spin_button_timer, (gpointer) spin);
+    }
+
+  gtk_spin_button_draw_arrow (spin, click_child);
+}
+
 static gint
 gtk_spin_button_button_press (GtkWidget      *widget,
 			      GdkEventButton *event)
@@ -937,7 +1008,6 @@ gtk_spin_button_button_press (GtkWidget      *widget,
 	{
 	  if (!GTK_WIDGET_HAS_FOCUS (widget))
 	    gtk_widget_grab_focus (widget);
-	  gtk_grab_add (widget);
 	  spin->button = event->button;
 	  
 	  if (GTK_ENTRY (widget)->editable)
@@ -945,65 +1015,21 @@ gtk_spin_button_button_press (GtkWidget      *widget,
 	  
 	  if (event->y <= widget->requisition.height / 2)
 	    {
-	      spin->click_child = GTK_ARROW_UP;
 	      if (event->button == 1)
-		{
-		 gtk_spin_button_real_spin (spin, 
-					    spin->adjustment->step_increment);
-		  if (!spin->timer)
-		    {
-		      spin->timer_step = spin->adjustment->step_increment;
-		      spin->need_timer = TRUE;
-		      spin->timer = gtk_timeout_add 
-			(SPIN_BUTTON_INITIAL_TIMER_DELAY, 
-			 (GtkFunction) gtk_spin_button_timer, (gpointer) spin);
-		    }
-		}
+		start_spinning (spin, GTK_ARROW_UP, spin->adjustment->step_increment);
 	      else if (event->button == 2)
-		{
-		 gtk_spin_button_real_spin (spin, 
-					    spin->adjustment->page_increment);
-		  if (!spin->timer) 
-		    {
-		      spin->timer_step = spin->adjustment->page_increment;
-		      spin->need_timer = TRUE;
-		      spin->timer = gtk_timeout_add 
-			(SPIN_BUTTON_INITIAL_TIMER_DELAY, 
-			 (GtkFunction) gtk_spin_button_timer, (gpointer) spin);
-		    }
-		}
-	      gtk_spin_button_draw_arrow (spin, GTK_ARROW_UP);
+		start_spinning (spin, GTK_ARROW_UP, spin->adjustment->page_increment);
+	      else
+		spin->click_child = GTK_ARROW_UP;
 	    }
 	  else 
 	    {
-	      spin->click_child = GTK_ARROW_DOWN;
 	      if (event->button == 1)
-		{
-		  gtk_spin_button_real_spin (spin,
-					     -spin->adjustment->step_increment);
-		  if (!spin->timer)
-		    {
-		      spin->timer_step = spin->adjustment->step_increment;
-		      spin->need_timer = TRUE;
-		      spin->timer = gtk_timeout_add 
-			(SPIN_BUTTON_INITIAL_TIMER_DELAY, 
-			 (GtkFunction) gtk_spin_button_timer, (gpointer) spin);
-		    }
-		}      
+		start_spinning (spin, GTK_ARROW_DOWN, spin->adjustment->step_increment);
 	      else if (event->button == 2)
-		{
-		  gtk_spin_button_real_spin (spin,
-					     -spin->adjustment->page_increment);
-		  if (!spin->timer) 
-		    {
-		      spin->timer_step = spin->adjustment->page_increment;
-		      spin->need_timer = TRUE;
-		      spin->timer = gtk_timeout_add 
-			(SPIN_BUTTON_INITIAL_TIMER_DELAY, 
-			 (GtkFunction) gtk_spin_button_timer, (gpointer) spin);
-		    }
-		}
-	      gtk_spin_button_draw_arrow (spin, GTK_ARROW_DOWN);
+		start_spinning (spin, GTK_ARROW_DOWN, spin->adjustment->page_increment);
+	      else
+		spin->click_child = GTK_ARROW_DOWN;
 	    }
 	  return TRUE;
 	}
@@ -1030,13 +1056,7 @@ gtk_spin_button_button_release (GtkWidget      *widget,
     {
       guint click_child;
 
-      if (spin->timer)
-	{
-	  gtk_timeout_remove (spin->timer);
-	  spin->timer = 0;
-	  spin->timer_calls = 0;
-	  spin->need_timer = FALSE;
-	}
+      gtk_spin_button_stop_spinning (spin);
 
       if (event->button == 3)
 	{
@@ -1064,11 +1084,10 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 		}
 	    }
 	}		  
-      gtk_grab_remove (widget);
       click_child = spin->click_child;
       spin->click_child = 2;
-      spin->button = 0;
       gtk_spin_button_draw_arrow (spin, click_child);
+
       return TRUE;
     }
   else
@@ -1317,7 +1336,7 @@ gtk_spin_button_key_release (GtkWidget   *widget,
   GtkSpinButton *spin;
 
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
-  
+
   spin = GTK_SPIN_BUTTON (widget);
   
   spin->ev_time = event->time;
@@ -1588,7 +1607,7 @@ gtk_spin_button_new_with_range (gdouble min,
 
   spin = gtk_type_new (GTK_TYPE_SPIN_BUTTON);
 
-  adj = gtk_adjustment_new (min, min, max, step, 10 * step, step);
+  adj = gtk_adjustment_new (min, min, max, step, 10 * step, 0);
 
   if (fabs (step) >= 1.0 || step == 0.0)
     digits = 0;
@@ -1924,9 +1943,13 @@ gtk_spin_button_set_numeric (GtkSpinButton  *spin_button,
 {
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
 
-  spin_button->numeric = (numeric != 0);
+  numeric = numeric != FALSE;
 
-  g_object_notify (G_OBJECT (spin_button), "numeric");
+  if (spin_button->numeric != numeric)
+    {
+       spin_button->numeric = numeric;
+       g_object_notify (G_OBJECT (spin_button), "numeric");
+    }
 }
 
 /**
@@ -1960,9 +1983,14 @@ gtk_spin_button_set_wrap (GtkSpinButton  *spin_button,
 {
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
 
-  spin_button->wrap = (wrap != 0);
+  wrap = wrap != FALSE; 
+
+  if (spin_button->wrap != wrap)
+    {
+       spin_button->wrap = (wrap != 0);
   
-  g_object_notify (G_OBJECT (spin_button), "wrap");
+       g_object_notify (G_OBJECT (spin_button), "wrap");
+    }
 }
 
 /**
